/*
 * network_impl.c
 *
 *  Created on: 01.07.2010
 *      Author: AndreR
 */

#include "stm32f746xx.h"
#include "constants.h"

#include "main/network.h"

#include "util/console.h"
#include "util/init_hal.h"
#include "util/intercom.h"

#include "hal/buzzer.h"
#include "hal/kicker.h"
#include "hal/motors.h"
#include "hal/led.h"

#include "config_main.h"

#include <string.h>

static uint8_t sxTx[SX1280_BUF_SIZE_RXTX] __attribute__((aligned(16), section(".dtc")));
static uint8_t sxRx[SX1280_BUF_SIZE_RXTX] __attribute__((aligned(16), section(".dtc")));

void DMA1_Stream3_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	SPILowDmaRxInterrupt(&network.wifi.spi);

	CH_IRQ_EPILOGUE();
}

void EXTI1_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	EXTI->PR = (1 << 1);

	SX1280DataIRQ(&network.wifi);

	CH_IRQ_EPILOGUE();
}

void EXTI15_10_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	EXTI->PR = (1 << 11);

	SX1280BusyIRQ(&network.wifi);

	CH_IRQ_EPILOGUE();
}

static void networkIntercomCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_NETWORK_NORMAL)
		NetworkSendPacket((PacketHeader*)pData, pData+PACKET_HEADER_SIZE, dataLength-PACKET_HEADER_SIZE);

	if(pHeader->command == IC_NETWORK_RELIABLE)
		NetworkSendPacketReliable((PacketHeader*)pData, pData+PACKET_HEADER_SIZE, dataLength-PACKET_HEADER_SIZE);
}

void NetworkImplInit()
{
	// SPI2 init
	RCC->APB1ENR |= RCC_APB1ENR_SPI2EN;

	NVICEnableIRQ(DMA1_Stream3_IRQn, IRQL_NRF24);

	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 5;
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_25MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOC, GPIO_PIN_2 | GPIO_PIN_3, &gpioInit);
	GPIOInit(GPIOD, GPIO_PIN_3, &gpioInit);

	// SX1280 busy and irq lines
	NVICEnableIRQ(EXTI1_IRQn, IRQL_NRF24);

	NVICEnableIRQ(EXTI15_10_IRQn, IRQL_NRF24);

	// SX1280 setup
	static uint8_t rxData[NRF24_RX_SIZE];
	static uint8_t txData[NRF24_TX_SIZE];
	RFQueueInit(&network.queue, txData, NRF24_TX_SIZE, rxData, NRF24_RX_SIZE, 32, 68);

	SX1280Data data;
	data.pTxBuf = sxTx;
	data.pRxBuf = sxRx;
	data.spiData.csPin = GPIO_PIN_4;
	data.spiData.pCSPort = GPIOA;
	data.spiData.pRegister = SPI2;
	data.spiData.pDma = DMA1;
	data.spiData.dmaChRx = 3;
	data.spiData.dmaChTx = 4;
	data.spiData.prescaler = SPI_CR1_BRDIV8;
	data.busyPin = GPIO_PIN_11;
	data.pBusyPort = GPIOF;
	data.irqPin = GPIO_PIN_1;
	data.pIrqPort = GPIOB;

	SX1280Init(&network.wifi, &data);

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &networkIntercomCallback;
	client.section = INTERCOM_SECTION_NETWORK;

	IntercomAddClient(client);
}

// this is the main network processing function
void NetworkImplProcessPacket(PacketHeader* pHeader, uint8_t* pBuf, uint16_t dataLength)
{
	switch(pHeader->section)
	{
		case SECTION_SYSTEM:
		{
			switch(pHeader->cmd)
			{
				case CMD_SYSTEM_CONSOLE_COMMAND:
				{
//					SystemConsoleCommand* pCmd = (SystemConsoleCommand*)pBuf;

//					if(pCmd->target == INTERCOM_ADDRESS_MAIN)
					{
						// process onboard
						uint8_t* pData;
						uint32_t length = dataLength-sizeof(SystemConsoleCommand);
						int16_t result = FifoLinReserve(&console.rxFifo, length, &pData);
						if(result)
							return;

						memcpy(pData, pBuf+sizeof(SystemConsoleCommand), length);

						FifoLinCommit(&console.rxFifo, length);
					}
//					else
//					{
//						if(cli.intercomFwd.inUse == 0)
//						{
//							cli.intercomFwd.bufSize = dataLength-sizeof(SystemConsoleCommand);
//
//							if(cli.intercomFwd.bufSize <= CLI_INTERCOM_BUF_SIZE)
//							{
//								cli.intercomFwd.inUse = 1;
//								cli.intercomFwd.target = pCmd->target;
//
//								memcpy(cli.intercomFwd.buf, pBuf+sizeof(SystemConsoleCommand), cli.intercomFwd.bufSize);
//							}
//						}
//					}
				}
				break;
			}
		}
		break;
		case SECTION_BOOTLOADER:
		{
//			if(pHeader->cmd == CMD_BOOTLOADER_CHECK)
//			{
//				IntercomHeader iHeader;
//				iHeader.address = INTERCOM_ADDRESS_MEDIA;
//				iHeader.section = INTERCOM_SECTION_NETWORK;
//				iHeader.command = 1;
//
//				IntercomReliableSend(&iHeader, 0, 0);
//			}
//			else
//			{
//				IntercomHeader iHeader;
//				iHeader.address = INTERCOM_ADDRESS_MEDIA;
//				iHeader.section = INTERCOM_SECTION_NETWORK;
//				iHeader.command = 0;
//
//				IntercomSend(&iHeader, dataLength+PACKET_HEADER_SIZE, pHeader);
//			}
		}
		break;

#ifdef ENV_RUN
		case SECTION_CONFIG:
		{
			ConfigNetworkInput(pHeader, pBuf, dataLength);
		}
		break;
#endif
		default: break;
	}
}
